"""Data Configuration Util"""
from ftpsync.constants import Common
from datatransfer.rest_client import RestClient
from ftpsync.settings import (job_user, job_password, config_url, process_log_url)


class DataConfigUtil:
    """Data configuration DAO"""

    @staticmethod
    def get_config_with_ftp_credentials(interval):
        res = RestClient.get(url=config_url,
                             auth={'user': job_user,
                                   'password': job_password}).json()
        results = []
        if res.get('status') == 'success':
            results = res['data']
        return results

    @staticmethod
    def get_process_status_by_config_id(config_id):
        file_names = []
        res = RestClient.get(url=process_log_url + "config/" + str(config_id),
                             auth={'user': job_user,
                                   'password': job_password}).json()
        if res.get('status') == 'success':
            for item in res['data']:
                file_names.append(item.get('source_file'))
        return file_names

    @staticmethod
    def add_process_key_status(config_id, file):
        inserted_id = None
        input = dict()
        input['config_id'] = config_id
        input['process_key'] = str(config_id) + "-" + file
        input['source_file'] = file
        input['status'] = Common.process_status_0.value
        res = RestClient.post(url=process_log_url,
                              auth={'user': job_user,
                                    'password': job_password},
                              data=input).json()
        if res.get('status') == 'success':
            inserted_id = res.get('data')['id']
        return inserted_id

    @staticmethod
    def update_process_key_status(process_log_id, success):
        input_msg = dict()
        input_msg['status'] = success['status']
        input_msg['status_msg'] = success['msg']
        input_msg['dest_file'] = success['dest_path']
        res = RestClient.put(url=process_log_url + str(process_log_id) + "/",
                             auth={'user': job_user,
                                   'password': job_password},
                             data=input_msg).json()
        return res



"""Rest Client using requests module"""
import requests


class RestClient:
    """Rest Client"""

    @staticmethod
    def get(url, headers=None, auth=None, query_params=None):
        """Get method"""
        response = ""
        if auth is not None:
            response = requests.get(url, params=query_params, headers=headers,
                                    auth=(auth['user'], auth['password']))
        else:
            response = requests.get(url, params=query_params, headers=headers)
        return response

    @staticmethod
    def post(url, headers=None, auth=None, data=None):
        """Post method"""
        response = ""
        if auth is not None:
            response = requests.post(url, json=data, headers=headers,
                                     auth=(auth['user'], auth['password']))
        else:
            response = requests.post(url, json=data, headers=headers)
        return response

    @staticmethod
    def put(url, headers=None, auth=None, data=None):
        """Put method"""
        response = ""
        if auth is not None:
            response = requests.put(url, json=data, headers=headers,
                                     auth=(auth['user'], auth['password']))
        else:
            response = requests.put(url, json=data, headers=headers)
        return response




# -*- coding: utf-8 -*-
import os
import time
import requests
import boto3
import pickle
import collections
import subprocess
from gzip import GzipFile
from io import BytesIO
from scorefastlib.common.rest_client import RestClient
from scorefastlib.common.constants import Constant


def list_dataset():
    """ list dataset """
    AUTH_TOKEN = os.environ.get('AUTH_TOKEN', '')
    assert AUTH_TOKEN
    dataset_ret = RestClient.get(
        url=Constant.dataset_url.value,
        headers={'Authorization': 'Token ' + AUTH_TOKEN}).json()
    if dataset_ret.get('status') != 'success':
        raise Exception("Failed to retrieve the list of dataset")
    return dataset_ret.get('data')

def read_dataset(dataset_id):
    """
    read the dataset

    :param dataset_id: The dataset id from platform
    :return: returns dataset content
    """
    AUTH_TOKEN = os.environ.get('AUTH_TOKEN', '')
    assert AUTH_TOKEN
    dataset_url = os.path.join(Constant.dataset_url.value, dataset_id)
    dataset_ret = RestClient.get(
        url=dataset_url,
        headers={'Authorization': 'Token ' + AUTH_TOKEN}).json()
    if dataset_ret.get('status') != 'success':
        raise Exception("Failed to retrieve the list of dataset")
    data = dataset_ret.get('data')
    file_path = data.get('path')
    
    # due to deprecated api
    if not file_path.startswith('api'):
        file_path = os.path.join('api', file_path)
    file_download_path = os.path.join(Constant.root_url.value, file_path)

    file_path, ext = os.path.splitext(file_download_path)
    
    headers = {'Authorization': 'Token ' + AUTH_TOKEN}
    r = requests.get(file_download_path, allow_redirects=True, headers=headers)

    if ext == '.gz':
        bytestream = BytesIO(r.content)
        return GzipFile(None, 'rb', fileobj=bytestream).read().decode('utf-8')
    else:
        return r.content.decode('utf-8')

def export_dataset(file_path, name, header=0,
    ignored_cols='', response_col='', tags='',
    privacy=0, license='other', parser_type='CSV', storage_config_id=1):
    """
    export dataset from notebook to console

    :param file_path: file path
    :param name: The name of dataset
    :param header: 0: No header, 1: header exists
    :param ignored_cols: Columns separated by comma eg. 'C1,C2,C3'
    :param response_col: Column for response
    :param tags: The list of tag
    :param privacy: 0: private, 1: public
    :param license: license name
    :param parser_type: CSV or TSV only
    :param storage_config_id: The id of storage config from the platform
    :return: returns nothing
    """
    AUTH_TOKEN = os.environ.get('AUTH_TOKEN', '')
    assert AUTH_TOKEN

    data = _get_user_summary(token=AUTH_TOKEN)
    user_id = data.get('user_id')
    group_id = data.get('group_id', 1)

    # validate the name
    if not name:
        raise Exception("name is necessary")
    else:
        if parser_type == 'CSV':
            file_name = '{}.csv'.format(name)
        elif parser_type == 'TSV':
            file_name = '{}.tsv'.format(name)
        else:
            raise Exception("Not supporting data format {}. only support CSV or TSV".format(parser_type))

    # upload dataset to S3 after change the name
    if not file_path:
        raise Exception("Failed to find the file_path {}".format(file_path))

    uuid = int(time.time() * 1000000)
    file_size = os.path.getsize(file_path)
    s3_namespace = "{}/customer/{}/{}/{}".format(
        Constant.s3_environment.value,
        group_id,
        uuid,
        file_name)
    s3_client = boto3.client('s3')
    s3_client.upload_file(
        file_path, Constant.s3_bucket.value, s3_namespace)

    # get sample dataset
    sample_dataset = _parse_dataset(
        token=AUTH_TOKEN,
        storage_id=storage_config_id,
        url=s3_namespace)

    # get dataset meta
    dataset_meta = _get_dataset_meta(sample_dataset)

    request_params = {
        "header": header,
        "ignored_cols": ignored_cols,
        "response_col": response_col,
        "tags": tags,
        "vpredict": "1",
        "privacy": privacy,
        "license": license,
        "name": name,
        "file_array": [{"path": s3_namespace, "name": file_name, "fileSize": file_size}],
        "total_file_size": file_size,
        "parser_type": parser_type,
        "sample_data": sample_dataset,
        "meta": dataset_meta,
        "subtitle": "",
        "description": "",
        "storage_config_id": storage_config_id
    }
    res = _post_dataset(token=AUTH_TOKEN, request_params=request_params)
    return str(res)

def _post_dataset(token, request_params):
    """ post dataset """
    dataset_ret = RestClient.post(
        url=Constant.dataset_import_url.value,
        headers={'Authorization': 'Token ' + token},
        data=request_params).json()

    if dataset_ret.get('status') != 'success':
        raise Exception("Failed to post the dataset")
    return dataset_ret.get('pk')

def _get_dataset_meta(sample_dataset):
    """ get datatset meta information """
    dataset_meta = {}
    mapping_desc = {}
    mapping_type = {}
    header = sample_dataset.split("\n")[0]
    header_list = header.split(",")
    for i, column_name in enumerate(header_list):
        mapping_desc[str(i)] = ""
        mapping_type[str(i)] = "auto"
    dataset_meta['column_desc'] = mapping_desc
    dataset_meta['column_type'] = mapping_type
    return dataset_meta

def import_model(build_id):
    """
    read the python model

    :param build_id: build id from the platform
    :return: returns pickle model
    """
    AUTH_TOKEN = os.environ.get('AUTH_TOKEN', '')
    assert AUTH_TOKEN

    headers = {'Authorization': 'Token ' + AUTH_TOKEN}
    url = os.path.join(Constant.model_download_url.value, "{}".format(build_id))
    r = requests.get(url, allow_redirects=True, headers=headers)
    file_path = "/tmp/{}.pkl".format(build_id)
    open(file_path, 'wb').write(r.content)
    loaded_model = pickle.load(open(file_path, 'rb'))
    if os.path.exists(file_path):
        os.remove(file_path)
    else:
        raise Exception("Failed to retrieve the model")
    return loaded_model

def _parse_dataset(token, storage_id, url):
    """ parse dataset """
    request_params = {
        "file_path": url,
        "range_start": 0,
        "range_end": 10485760,
        "storage_id": storage_id
    }

    dataset_parse_ret = RestClient.post(
        url=Constant.dataset_parse_url.value,
        headers={'Authorization': 'Token ' + token},
        data=request_params).json()

    if dataset_parse_ret.get('status') != 'success':
        raise Exception("Failed to parse the dataset")
    return dataset_parse_ret.get('data')

def _get_user_summary(token):
    """ get user meta information """
    user_summary_ret = RestClient.get(
        url=Constant.user_summary_url.value,
        headers={'Authorization': 'Token ' + token}).json()
    if user_summary_ret.get('status') != 'success':
        raise Exception("Failed to retrieve the user summary")
    return user_summary_ret.get('data')

def deploy_model(model, name, dataset_id=None, algorithm=None, build_id=None):
    """
    deploy the model

    :param model: python classifier
    :param name: The name of the model will show in the platform
    :param dataset_id: The id of dataset
    :param algorithm: Custom algorithm name
    :param build_id: build id in order to predict, otherwise, it will be random
    :return: returns nothing
    """
    AUTH_TOKEN = os.environ.get('AUTH_TOKEN', '')
    assert AUTH_TOKEN

    data = _get_user_summary(token=AUTH_TOKEN)
    user_id = data.get('user_id')
    group_id = data.get('group_id', 1)
    
    # create pickle
    directory = os.path.join('/tmp', str(group_id))
    if not os.path.exists(directory):
        os.makedirs(directory)
    file_name = '{}.pkl'.format(name)
    file_path = '{}/{}'.format(directory, file_name)
    if os.path.exists(file_path):
        os.remove(file_path)
    pickle.dump(model, open(file_path, 'wb'))

    # upload the model to S3
    uuid = int(time.time() * 1000000)
    file_size = os.path.getsize(file_path)
    s3_namespace = "{}/customer/{}/{}/{}".format(
        Constant.s3_environment.value,
        group_id,
        uuid,
        file_name)
    s3_client = boto3.client('s3')
    s3_client.upload_file(
        file_path, Constant.s3_bucket.value, s3_namespace)
    
    # ping API end point to set
    params = {
        "name": name,
        "type": 3,
        "dataset_id": dataset_id,
        "algorithm": algorithm,
        "file_array": [{
            "name": file_name,
            "file_size": file_size,
            "path": s3_namespace}]
    }

    if build_id:
        params['build_id'] = build_id

    model_import_ret = RestClient.post(
        url=Constant.model_import_url.value,
        headers={'Authorization': 'Token ' + AUTH_TOKEN},
        data=params).json()
    if model_import_ret.get('status') != 'success':
        raise Exception("Failed to deploy the model")

def sync_to_cloud():
    """
    sync_to_cloud
    """
    AUTH_TOKEN = os.environ.get('AUTH_TOKEN', '')
    assert AUTH_TOKEN

    data = _get_user_summary(token=AUTH_TOKEN)
    user_id = data.get('user_id')
    dest = "s3://{}/notebook/{}/{}".format(
        Constant.s3_bucket.value,
        Constant.s3_environment.value,
        user_id)
    subprocess.run(["aws", "s3", "sync", "/work_directory", dest], check=True)

def deploy_model_(model, name, dataset_id, algorithm=None, build_id=None):
    """
    deploy the model
    :param model: python model
    :param name: The name of the model will show in the platform
    :param dataset_id: The id of dataset, used for building the model, from platform
    :param algorithm: Custom algorithm name
    :param build_id: build id in order to predict, otherwise, it will be random
    :return: returns nothing
    """
    AUTH_TOKEN = os.environ.get('AUTH_TOKEN', '')
    assert AUTH_TOKEN

    data = _get_user_summary(token=AUTH_TOKEN)
    user_id = data.get('user_id')
    group_id = data.get('group_id', 1)
    
    # create pickle
    directory = os.path.join('/tmp', str(group_id))
    if not os.path.exists(directory):
        os.makedirs(directory)
    file_name = '{}.pkl'.format(name)
    file_path = '{}/{}'.format(directory, file_name)
    if os.path.exists(file_path):
        os.remove(file_path)
    pickle.dump(model, open(file_path, 'wb'))

    # upload the model to S3
    uuid = int(time.time() * 1000000)
    file_size = os.path.getsize(file_path)
    s3_namespace = "{}/customer/{}/{}/{}".format(
        Constant.s3_environment.value,
        group_id,
        uuid,
        file_name)
    s3_client = boto3.client('s3')
    s3_client.upload_file(
        file_path, Constant.s3_bucket.value, s3_namespace)
    
    # ping API end point to set
    params = {
        "name": name,
        "type": 3,
        "dataset_id": dataset_id,
        "algorithm": algorithm,
        "file_array": [{
            "name": file_name,
            "file_size": file_size,
            "path": s3_namespace}]
    }

    if build_id:
        params['build_id'] = build_id

    model_import_ret = RestClient.post(
        url=Constant.model_import_url.value,
        headers={'Authorization': 'Token ' + AUTH_TOKEN},
        data=params).json()
    if model_import_ret.get('status') != 'success':
        raise Exception("Failed to deploy the model")

def list_project():
    """ list projects """
    AUTH_TOKEN = os.environ.get('AUTH_TOKEN', '')
    assert AUTH_TOKEN
    project_ret = RestClient.get(
        url=Constant.project_url.value,
        headers={'Authorization': 'Token ' + AUTH_TOKEN}).json()
    if project_ret.get('status') != 'success':
        raise Exception("Failed to retrieve the list of projects")
    return project_ret.get('data')

def get_project_content(project_id, datasets=True, models=True, files=True):
    """ accessing content of a project
    :param project_id: The project id from platform
    :param datasets: boolean, optional, default True. Set to false to ignore list of the datasets in the project
    :param models: boolean, optional, default True. Set to false to ignore list of the models in the project
    :param files: boolean, optional, default True. Set to false to ignore list of the files in the project

    :return: returns contents of the project in a python dictionary format
    """
    AUTH_TOKEN = os.environ.get('AUTH_TOKEN', '')
    assert AUTH_TOKEN

    response = {}
    response['project_id'] = project_id
    project_url = os.path.join(Constant.project_url.value, str(project_id))
    project_ret = RestClient.get(
        url=project_url,
        headers={'Authorization': 'Token ' + AUTH_TOKEN}).json()

    if project_ret.get('status') != 'success':
        raise Exception("Failed to access project_id : {}".format(project_id))

    response['project_name'] = project_ret['data']['name']

    if datasets:

        datasets_url = os.path.join(project_url,'dataset')
        dataset_ret = RestClient.get(
        url=datasets_url,
        headers={'Authorization': 'Token ' + AUTH_TOKEN}).json()
        if dataset_ret.get('status') != 'success':
            raise Exception("Failed to access project_id : {}'s datasets".format(project_id))

        response['datasets'] = dataset_ret.get('data')

    if models:

        model_url = os.path.join(project_url,'model')
        model_ret = RestClient.get(
        url=model_url,
        headers={'Authorization': 'Token ' + AUTH_TOKEN}).json()
        if model_ret.get('status') != 'success':
            raise Exception("Failed to access project_id : {}'s models".format(project_id))

        response['models'] = model_ret.get('data')

    if files:

        file_url = os.path.join(project_url,'file')
        file_ret = RestClient.get(
        url=file_url,
        headers={'Authorization': 'Token ' + AUTH_TOKEN}).json()
        if file_ret.get('status') != 'success':
            raise Exception("Failed to access project_id : {}'s files".format(project_id))

        response['files'] = file_ret.get('data')

    return response


def download_file_from_project(project_id, file_name=None):
    """
    download project files into local resource
    :param project_id: Id of the project assosiated with the requested file from platform
    :param file_name: string, default None, optional. The file name from platform. If not passed all the files in the project will be downloaded

    :return: downloaded files' path in local resource
    """
    AUTH_TOKEN = os.environ.get('AUTH_TOKEN', '')
    assert AUTH_TOKEN

    project_file_url = os.path.join(Constant.project_url.value, str(project_id), 'file')
    project_file_ret = RestClient.get(
        url=project_file_url,
        headers={'Authorization': 'Token ' + AUTH_TOKEN}).json()

    if project_file_ret.get('status') != 'success':
        raise Exception("Failed to access project_id : {}'s files".format(project_id))

    if file_name:
        file_download_path = ''
        file_size = 0
        file_list = project_file_ret.get('data')
        for file in file_list:
            if file['name'] == file_name:
                file_download_path = file.get('path')
                file_size = file.get('file_size')
                break

        if len(file_download_path) == 0:
            raise Exception("Failed to find file {} in project {}".format(file_name, project_id))

        file_download_url = os.path.join(Constant.root_url.value, file_download_path)
        local_path = _download_file(project_id, file_download_url, file_name, file_size)
        return 'successfully downloaded file to {}'.format(local_path)

    local_paths = []
    file_list = project_file_ret.get('data')
    for file in file_list:
        file_download_path = file.get('path')
        file_size = file.get('file_size')
        file_name = file.get('name')
        file_download_url = os.path.join(Constant.root_url.value, file_download_path)
        local_path = _download_file(project_id, file_download_url, file_name, file_size)
        local_paths.append(local_path)
    directory = '/tmp/{}'.format(str(project_id))

    return 'successfully downloaded project {} files to {}'.format(project_id, directory)

def _download_file(project_id, file_download_url, file_name, file_size):
    """ download a file from project"""
    AUTH_TOKEN = os.environ.get('AUTH_TOKEN', '')
    assert AUTH_TOKEN

    file_download_ret = RestClient.get(
        url=file_download_url,
        headers={'Authorization': 'Token ' + AUTH_TOKEN})
    if str(file_download_ret.status_code) != '200':
        raise Exception("Failed to download file {}".format(file_name))

    directory = os.path.join('/tmp', str(project_id))
    if not os.path.exists(directory):
        os.makedirs(directory)
    local_path = '{}/{}'.format(directory, file_name)
    with open(local_path, 'w+b') as local:
        local.write(file_download_ret.content)

    if file_size != os.path.getsize(local_path):
        raise Exception("Failed to download file {}".format(file_name))

    return local_path

def upload_file_to_project(project_id, file_path, file_name):
    """
    upload a file from local resurce to a project on platform

    :param project_id: Id of the destination project from platform
    :param file_path: string. The local file path
    :param file_name: string. Name of the file on the platform

    :return: id of the datafile
    """
    AUTH_TOKEN = os.environ.get('AUTH_TOKEN', '')
    assert AUTH_TOKEN

    data = _get_user_summary(token=AUTH_TOKEN)
    user_id = data.get('user_id')
    group_id = data.get('group_id', 1)
    # upload the file to S3
    uuid = int(time.time() * 1000000)
    file_size = os.path.getsize(file_path)
    s3_namespace = "{}/customer/{}/{}/{}".format(
        Constant.s3_environment.value,
        group_id,
        uuid,
        file_name)
    s3_client = boto3.client('s3')
    s3_client.upload_file(
        file_path, Constant.s3_bucket.value, s3_namespace)

    headers = {'Authorization': 'Token ' + AUTH_TOKEN}
    payload = {"project_id":project_id,
                "path":s3_namespace,
                "file_size":file_size,
                "name":file_name}

    file_ret = RestClient.post(
            url=Constant.file_url.value,
            headers=headers,
            data=payload).json()

    if file_ret.get('status') != 'success':
        raise Exception("Failed to upload_file {} to project_id {}".format(file_name, project_id))

    return 'successfully uploaded file {} to project_id {} with datafile id {}'.\
                                format(file_name,project_id,file_ret['data']['id'])

def sync_to_cloud__():
    """
    sync_to_cloud
    """
    AUTH_TOKEN = os.environ.get('AUTH_TOKEN', '')
    assert AUTH_TOKEN

    data = _get_user_summary(token=AUTH_TOKEN)
  
    user_id = data.get('user_id')
    dest = "s3://{}/notebook/{}/{}".format(
        Constant.s3_bucket.value,
        Constant.s3_environment.value,
        user_id)
    
    subprocess.run(["aws", "s3", "sync", "/work_directory", dest, "--exclude", '*', "--include", '*.ipynb'], check=True)
        

if __name__ == '__main__':
    read_dataset("1377")
    set_model("", name="finalized_model")



job_user = "ravisharma@scoredata.com"
job_password = "ravi@score1"

config_url = 'http://develop.scoredata.com/dw/client/config'
process_log_url = "http://develop.scoredata.com/dw/datatransfer/process_log/"



"""Util module"""
import os
import datetime
from ftplib import FTP
from fluent import event
import pysftp


class FTPUtility:
    """Common methods"""

    @staticmethod
    def log_event(tag, data_dict):
        """ fluentd event generator """
        data_dict['event_datetime'] = str(datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"))
        event.Event(tag, data_dict)

    @staticmethod
    def delete_file(file):
        try:
            os.remove(file)
        except:
            pass

    @staticmethod
    def get_ftp_connection(host, user, password):
        hostport = host.split(":")
        host = hostport[0]
        port = int(hostport[1]) if len(hostport) == 2 else 21
        ftp = FTP(host=host, port=port)
        ftp.login(user, password)
        return ftp

    @staticmethod
    def get_sftp_connection(host, user, password):
        hostport = host.split(":")
        host = hostport[0]
        port = int(hostport[1]) if len(hostport) == 2 else 20
        cinfo = {'host': host, 'username': user, 'password': password,
                 'port': port}
        cnopts = pysftp.CnOpts()
        cnopts.hostkeys = None
        sftp = pysftp.Connection(**cinfo, cnopts=cnopts)
        return sftp




# -*- coding: utf-8 -*-
import requests


class RestClient(object):
    """ RESTful Client """
    @staticmethod
    def get(url, headers={}, auth={}, query_params={}):
        response = ""
        if len(auth) > 0:
            response = requests.get(url,  params=query_params, headers=headers, auth=(
                auth['user'], auth['password']))
        else:
            response = requests.get(url, params=query_params, headers=headers)
        return response

    @staticmethod
    def post(url, headers={}, auth={}, data={}):
        response = ""
        if len(auth) > 0:
            response = requests.post(url, json=data, headers=headers, auth=(
                auth['user'], auth['password']))
        else:
            response = requests.post(url, json=data, headers=headers)
        return response



"""
FTP Driver
"""

import os
import time
from datetime import datetime

from apscheduler.schedulers.background import BackgroundScheduler
from fluent import sender

from ftpsync.constants import Common
from ftpsync.ftp_transfer import FTPTransfer

# for remote fluent
sender.setup(Common.logger_app_name.value, host='localhost', port=24224)


def minute():
    """Run job at every 15 minutes"""
    FTPTransfer.process(Common.interval_minute.value)


def hour():
    """Run job on every hour"""
    FTPTransfer.process(Common.interval_hourly.value)


def day():
    """Run job once in a day at 1 o'clock"""
    FTPTransfer.process(Common.interval_daily.value)


def week():
    """Run job on every Monday at 2 o'clock"""
    FTPTransfer.process(Common.interval_weekly.value)


def month():
    """Run job on every month's 1st at 3 o'clock """
    FTPTransfer.process(Common.interval_monthly.value)


if __name__ == '__main__':

    print(datetime.now())
    scheduler = BackgroundScheduler()
    scheduler.add_job(minute, 'interval', id='min', minutes=15, next_run_time=datetime.now())
    scheduler.add_job(hour, 'cron', id='hour', hour='*/1')
    scheduler.add_job(day, 'cron', id='day', hour='1')
    scheduler.add_job(week, 'cron', id='week', day_of_week='mon', hour=2)
    scheduler.add_job(month, 'cron', id='month', day='1', hour=3)

    scheduler.start()
    print('Press Ctrl+{0} to exit'.format('Break' if os.name == 'nt' else 'C'))

    try:
        # This is here to simulate application activity (which keeps the main thread alive).
        while True:
            time.sleep(2)
    except (KeyboardInterrupt, SystemExit):
        # Not strictly necessary if daemonic mode is enabled but should be done if possible
        scheduler.shutdown()



"""Watcher"""
import logging
import time

from watchdog.events import FileSystemEventHandler
from watchdog.observers import Observer

from datatransfer import constants
from datatransfer.local_data_transfer import LocalDataTransfer


class Watcher:
    """File event based watcher"""

    def __init__(self):
        self.observer = Observer()
        self.logger = logging.getLogger(
            constants.Constant.logger_module_name.value)

    def run(self):
        """Start observer"""
        event_handler = Handler()
        for dir_path in constants.SavedObj.config[constants.Config.dir_path_list.value]:
            self.observer.schedule(event_handler, dir_path, recursive=True)
        self.observer.start()
        try:
            while True:
                time.sleep(5)
        except Exception as ex:
            self.observer.stop()
            self.logger.error(ex)

        self.observer.join()


class Handler(FileSystemEventHandler):
    """File system handler"""

    @staticmethod
    def on_any_event(event):
        """Implicitly called on any file event"""

        logger = logging.getLogger(constants.Constant.logger_module_name.value)
        if event.is_directory:
            return None
        elif event.event_type == 'created':
            # Take any action here when a file is first created.
            logger.info("received created event - %s.", event.src_path)
            df = LocalDataTransfer()
            df.upload(event.src_path)



import enum


class Common(enum.Enum):

    interval_minute = "minute"
    interval_hourly = "hourly"
    interval_daily = "daily"
    interval_weekly = "weekly"
    interval_monthly = "monthly"

    process_status_0 = "INPROGRESS"
    process_status_1 = "FAILED"
    process_status_2 = "COMPLETED"

    logger_app_name = "ftpsync"
    logger_tag_warn = "WARN"
    logger_tag_info = "INFO"
    logger_tag_error = "ERROR"




"""
S3 Processor
"""
import os
import socket
import time

import boto3
from boto3.s3.transfer import S3Transfer

from datatransfer.util import Utility
from ftpsync.constants import Common
from ftpsync.rest_driver import DataConfigUtil
from ftpsync.ftp_util import FTPUtility
from ftpsync.settings import ( job_user, job_password)


class S3Processor:
    """Takes care of S3 files upload and download"""

    @staticmethod
    def put_files(config, file):
        """Upload local files to S3"""
        success = {}
        local_file_path = ''
        compress_file = ''
        dest_path = ''
        try:
            s3_conf = Utility.__authenticate_request__(job_user,
                                                       job_password)
            access_key = s3_conf.get('access_key')
            secret_key = s3_conf.get('secret_key')
            bucket_name = s3_conf.get('bucket')
            date_format = s3_conf.get('namespace')[2:]
            client_id = str(config['client_id'])
            transfer = S3Transfer(boto3.client('s3', aws_access_key_id=access_key,
                                               aws_secret_access_key=secret_key))
            local_file_path = str(client_id) + "/" + str(config['config_id']) + "/" + file
            local_file_dir = local_file_path[:local_file_path.rfind('/')]
            # compress file
            compress_file = Utility.__compress_file__(local_file_path, local_file_dir, "gz")
            # upload file
            file_name = os.path.basename(compress_file)
            hostname_unixtime = socket.gethostname() + "_" + str(
                int(time.time()))
            file_name = hostname_unixtime + "_" + file_name
            dest_path = client_id + "/" + config['dest'] + "/" + date_format + "/" + file_name
            # upload file
            transfer.upload_file(compress_file, bucket_name, dest_path)
            # build success
            success['status'] = Common.process_status_2.value
            success['msg'] = "Success"
            success['source_path'] = file
            success['dest_path'] = dest_path

        except Exception as ex:
            success['status'] = Common.process_status_1.value
            success['msg'] = str(ex)
            success['source_path'] = file
            success['dest_path'] = dest_path
        finally:
            FTPUtility.delete_file(local_file_path)
            FTPUtility.delete_file(compress_file)
        return success

    def get_files(self, config):
        """Download files from S3"""
        s3_conf = Utility.__authenticate_request__(job_user, job_password)
        access_key = s3_conf.get('access_key')
        secret_key = s3_conf.get('secret_key')
        bucket_name = s3_conf.get('bucket')
        client = boto3.client('s3', aws_access_key_id=access_key,
                              aws_secret_access_key=secret_key)
        transfer = S3Transfer(client)
        # get list of files from s3
        list_files = []
        response = client.list_objects(Bucket=bucket_name,
                                       Prefix=config['client_id'] + "/" + config['source'])
        for content in response.get('Contents', []):
            list_files.append(content.get('Key'))

        processed_files_list = DataConfigUtil.get_process_status_by_config_id(config['config_id'])
        unprocessed_files = [x for x in list_files if x not in processed_files_list]
        files = []
        # download
        for unprocessed_file in unprocessed_files:
            f = unprocessed_file.replace("/", "|")
            transfer.download_file(bucket_name, unprocessed_file, f)
            files.append(unprocessed_file)

        return files



"""FTP Transfer main job"""

import time
import datetime
import copy

from ftpsync.constants import Common
from ftpsync.rest_driver import DataConfigUtil
from ftpsync.ftp_processor import FTPProcessor
from ftpsync.ftp_util import FTPUtility
from ftpsync.s3_processor import S3Processor
from ftpsync.sftp_processor import SFTPProcessor


class FTPTransfer:
    """Executes job basis on client configuration. Supports FTP and S3 data source"""

    @staticmethod
    def process(interval):
        """Process job on interval basis"""

        log_events = {}
        log_events['interval'] = interval
        log_events['job_id'] = str(int(time.time()))
        log_events['job_datetime'] = str(datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"))
        log_job_events = copy.deepcopy(log_events)
        log_job_events['job_status'] = 'started'
        try:

            FTPUtility.log_event(Common.logger_tag_info.value, log_job_events)
            config_list = DataConfigUtil.get_config_with_ftp_credentials(interval)
            if len(config_list) == 0:
                log_events['ftptransfer_config'] = 'None'
                FTPUtility.log_event(Common.logger_tag_info.value, log_events)

            for config in config_list:
                print(config)
                log_events['ftptransfer_config'] = config
                list_files = []
                try:
                    download_processor = FTPTransfer.get_processor(config['source_type'])
                    list_files = download_processor.get_files(config)
                    upload_processor = FTPTransfer.get_processor(config['dest_type'])
                except Exception as ex:
                    log_events['ftptransfer_download'] = str(ex)
                    FTPUtility.log_event(Common.logger_tag_error.value, log_events)
                    continue

                for file in list_files:
                    try:
                        # log file processing status
                        record_id = DataConfigUtil.add_process_key_status(config['config_id'],
                                                                             file)
                        if record_id is None:
                            continue
                        success = upload_processor.put_files(config, file)
                        log_events['ftptransfer_status'] = success
                        FTPUtility.log_event(Common.logger_tag_info.value, log_events)
                        DataConfigUtil.update_process_key_status(record_id, success)
                    except Exception as ex:
                        log_events['ftptransfer_upload'] = str(ex)
                        FTPUtility.log_event(Common.logger_tag_error.value, log_events)

        except Exception as ex:
            log_events['ftptransfer_process'] = str(ex)
            FTPUtility.log_event(Common.logger_tag_error.value, log_events)
        finally:
            log_job_events['job_status'] = 'completed'
            FTPUtility.log_event(Common.logger_tag_info.value, log_job_events)

    @staticmethod
    def get_processor(type):

        if type == 'FTP':
            return FTPProcessor()
        elif type == 'S3':
            return S3Processor()
        elif type == 'SFTP':
            return SFTPProcessor()
        else:
            raise Exception("ftp data source not supported")



"""Constants - Holds all the default and client configuration"""
import enum


class Config(enum.Enum):
    """Config - Contains client configuration values as an enum"""

    section_client_credential = "client.credential"
    client_credential_user = "user_name"
    client_credential_password = "password"

    section_data_location = "data.location"
    dir_path_list = "dir_path_list"

    section_log_location = "log.location"
    file_process_log = "file_process_log"
    application_log = "application_log"

    section_app_settings = "app.settings"
    compression_type = "compression_type"
    delete_file = "delete_file"


class Constant(enum.Enum):
    """Application configuration as an enum"""

    score_token_url = "http://console.scoredata.com/api/token/"
    score_s3_config_url = "http://console.scoredata.com/dw/logger"

    file_checkpoint_path = "file_checkpoint.log"
    app_log_path = "app.log"
    logger_module_name = "datatransfer"

    default_compression_type = "gz"
    default_s3_limit_mb = 10240

    exit_missing_credential = "Username and password not provided"
    exit_data_path_location = "Dir path missing"


class SavedObj:
    """Holds application and client configuration"""
    config = {}



"""Data Transfer Driver"""
import argparse
import configparser
import json
import logging.config
import sys
from logging.handlers import TimedRotatingFileHandler

from datatransfer import constants
from datatransfer.local_data_transfer import LocalDataTransfer
from datatransfer.watcher import Watcher


class Driver:
    """Application's main program. Client has to provide configuration as an
    argument"""

    def __init__(self, arguments):
        """Load client and application configuration"""

        config = configparser.ConfigParser()
        config.read(arguments.conf)

        # initialize logger
        self.logger = logging.getLogger(
            constants.Constant.logger_module_name.value)
        self.logger.setLevel(logging.DEBUG)
        formatter = logging.Formatter(
            '%(asctime)s - %(name)s - %(levelname)s - %(message)s')

        logger_handler = logging.StreamHandler(sys.stdout)
        logger_handler.setLevel(logging.INFO)
        logger_handler.setFormatter(formatter)
        self.logger.addHandler(logger_handler)
        app_log = ""
        try:
            app_log = config.get(constants.Config.section_log_location.value,
                                 constants.Config.application_log.value)
        except Exception:
            pass
        if len(app_log) == 0:
            app_log = constants.Constant.app_log_path.value

        time_log_handler = TimedRotatingFileHandler(app_log, when="D",
                                                    interval=1, backupCount=10)
        time_log_handler.setFormatter(formatter)
        time_log_handler.setLevel(logging.INFO)
        self.logger.addHandler(time_log_handler)

        file_process_log = ""
        try:
            file_process_log = config.get(
                constants.Config.section_log_location.value,
                constants.Config.file_process_log.value)
        except Exception:
            pass
        if len(file_process_log) == 0:
            file_process_log = constants.Constant.file_checkpoint_path.value

        constants.SavedObj.config[
            constants.Config.file_process_log.value] = file_process_log

        user_name = config.get(
            constants.Config.section_client_credential.value,
            constants.Config.client_credential_user.value)
        password = config.get(constants.Config.section_client_credential.value,
                              constants.Config.client_credential_password.value)
        if len(user_name) == 0 or len(password) == 0:
            self.logger.error("Missing username and password")
            sys.exit(constants.Constant.exit_missing_credential.value)

        constants.SavedObj.config[constants.Config.client_credential_user.value] = user_name
        constants.SavedObj.config[constants.Config.client_credential_password.value] = password

        constants.SavedObj.config[constants.Config.dir_path_list.value] = json.loads(
            config.get(constants.Config.section_data_location.value,
                       constants.Config.dir_path_list.value))
        if type(constants.SavedObj.config[constants.Config.dir_path_list.value]).__name__ != 'list':
            self.logger.error(
                "Provide list of dir name in list [] format in config.conf file")
            sys.exit(constants.Constant.exit_data_path_location.value)
        if len(constants.SavedObj.config[constants.Config.dir_path_list.value]) == 0:
            self.logger.error("Provide dir name in configuration file")
            sys.exit(constants.Constant.exit_data_path_location.value)

        compression_type = ""
        try:
            compression_type = config.get(
                constants.Config.section_app_settings.value,
                constants.Config.compression_type.value)
        except Exception:
            pass
        if len(compression_type) == 0:
            compression_type = constants.Constant.default_compression_type.value
        constants.SavedObj.config[
            constants.Config.compression_type.value] = compression_type
        delete_file = False
        try:
            delete_file = config.getboolean(
                constants.Config.section_app_settings.value,
                constants.Config.delete_file.value)
        except Exception as ex:
            delete_file = False
            self.logger.warning(ex)
        constants.SavedObj.config[
            constants.Config.delete_file.value] = delete_file

    def load(self):
        """Load existing data and initiate watcher for any updates on given directory list"""
        try:
            # add existing data
            self.load_data()
            # add watcher
            w = Watcher()
            w.run()
        except Exception as ex:
            self.logger.error(ex)
            sys.exit("Stopped")

    @staticmethod
    def load_data():
        """Iterate all the given list of directories"""
        file_transfer = LocalDataTransfer()
        for dir_path in constants.SavedObj.config[constants.Config.dir_path_list.value]:
            file_transfer.upload(dir_path)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--conf', action='store',
                        dest='conf',
                        help='Config file location.')

    args = parser.parse_args()

    if args.conf is None:
        parser.error("missing config file")
        sys.exit()
    driver = Driver(args)
    driver.load()



"""FTPProcessor"""
import os
from ftplib import error_perm

from ftpsync.constants import Common
from ftpsync.rest_driver import DataConfigUtil
from ftpsync.ftp_util import FTPUtility


class FTPProcessor:
    """Takes care of FTP files upload and download"""

    def download_file(self, file_path, config):
        """Download FTP files to local machine"""

        success = 0
        ftp = FTPUtility.get_ftp_connection(config['host_name'], config['username'],
                                            config['password'])
        ftp.set_pasv(True)
        dir_loc = '.'
        filename = file_path
        le = file_path.rfind('/')
        if le > 0:
            dir_loc = dir_loc + "/" + file_path[:le]
            filename = file_path[le + 1:]
        ftp.cwd(dir_loc)
        local_file_path = str(config['client_id']) + "/" + str(config['config_id']) + "/" + dir_loc
        filepath = os.path.join(local_file_path, filename)
        if not os.path.exists(local_file_path):
            os.makedirs(local_file_path)
        localfile = open(filepath, 'wb')
        ftp.retrbinary('RETR ' + filename, localfile.write, 1024)
        ftp.close()
        localfile.close()
        success = 1
        return success

    def get_files(self, config):
        """Get FTP files"""

        ftp = FTPUtility.get_ftp_connection(config['host_name'], config['username'],
                                            config['password'])
        ftp.cwd(config['source'])
        ftp.set_pasv(True)
        list_files = self.traverse(ftp, config['source'], [])
        ftp.close()
        # check this files process status
        processed_files_list = DataConfigUtil.get_process_status_by_config_id(config['config_id'])
        unprocessed_files = [x for x in list_files if x not in processed_files_list]
        files = []
        # download
        for unprocessed_file in unprocessed_files:
            success = self.download_file(unprocessed_file, config)
            if success == 1:
                files.append(unprocessed_file)
        return files

    def traverse(self, ftp, k, list_files, depth=0):
        """
        return a recursive listing of an ftp server contents (starting
        from the current directory)

        listing is returned as a recursive dictionary, where each key
        contains a contents of the subdirectory or None if it corresponds
        to a file.

        @param ftp: ftplib.FTP object
        """
        if depth > 10:
            return ['depth > 10']
        level = {}
        for entry in (path for path in ftp.nlst() if path not in ('.', '..')):
            try:
                ftp.cwd(entry)
                s = entry
                if (len(k) != 0):
                    s = k + '/' + entry

                level[entry] = self.traverse(ftp, s, list_files, depth + 1)
                ftp.cwd('..')
            except error_perm:
                level[entry] = None
                s = entry
                if (len(k) != 0):
                    s = k + '/' + entry
                    list_files.append(s)
        return list_files

    @staticmethod
    def put_files(config, file_path):
        """Upload local files to FTP server"""

        success = {}
        dest_path = ''
        local_file_name = ''
        try:
            local_file_name = file_path.replace("/", "|")
            ftp = FTPUtility.get_ftp_connection(config['host_name'], config['username'],
                                                config['password'])
            ftp.cwd(config['dest'])
            dir_path = os.path.dirname(file_path)
            file_name = os.path.basename(file_path)
            FTPProcessor.mk_ftp_dir(ftp, dir_path)
            dest_path = os.path.join(config['dest'], file_path)
            local_file = open(local_file_name, "rb")
            ftp.storbinary('STOR ' + file_name, local_file)
            ftp.quit()
            local_file.close()
            success['status'] = Common.process_status_2.value
            success['msg'] = "Success"
            success['source_path'] = file_path
            success['dest_path'] = dest_path
        except Exception as ex:
            success['status'] = Common.process_status_1.value
            success['msg'] = str(ex)
            success['source_path'] = file_path
            success['dest_path'] = dest_path
        finally:
            FTPUtility.delete_file(local_file_name)
        return success

    @staticmethod
    def mk_ftp_dir(ftp, dir_path):
        d_tmp = dir_path.split("/")
        for dir_loc in d_tmp:
            if FTPProcessor.ftp_directory_exists(ftp, dir_loc) is False:
                ftp.mkd(dir_loc)
            ftp.cwd(dir_loc)

    @staticmethod
    def ftp_directory_exists(ftp, dir_path):
        filelist = []
        ftp.retrlines('LIST', filelist.append)
        for file in filelist:
            if file.split()[-1] == dir_path and file.upper().startswith('D'):
                return True
        return False



"""SFTPProcessor"""
import os
from ftpsync.constants import Common
from ftpsync.rest_driver import DataConfigUtil
from ftpsync.ftp_util import FTPUtility


class SFTPProcessor:
    """Takes care of SFTP files upload and download"""

    def __init__(self):
        self.processed_files_list = []
        self.unprocessed_files_list = []

    def get_files(self, config):
        """Get SFTP files"""

        source_dir = os.path.join('/', config['source'])
        print(source_dir)
        local_dest_dir_prefix = str(config['client_id']) + "/" + str(config['config_id'])
        local_dest_dir = os.path.join(local_dest_dir_prefix, config['source'])

        # check this files process status
        self.processed_files_list = DataConfigUtil.get_process_status_by_config_id(config['config_id'])
        sftp = FTPUtility.get_sftp_connection(config['host_name'], config['username'],
                                              config['password'])
        self._sync_r(sftp, source_dir, local_dest_dir)
        sftp.close()
        return self.unprocessed_files_list

    def _sync_r(self, sftp, remote_dir, local_dir):
        """
            Recursively sync the sftp contents starting at remote dir to the local dir and return the number of files synced.
            :param sftp:        Connection to the sftp server.
            :param remote_dir:  Remote dir to start sync from.
            :param local_dir:   To sync to.
            :return             The number of files synced.
            """
        files_synced = 0
        for item in sftp.listdir(remote_dir):
            remote_dir_item = os.path.join(remote_dir, item)
            local_dir_item = os.path.join(local_dir, item)
            if sftp.isfile(remote_dir_item):
                if not os.path.exists(local_dir):
                    os.makedirs(local_dir)
                if self._should_sync_file(sftp, remote_dir_item, local_dir_item):
                    if remote_dir_item not in self.processed_files_list:
                        print('sync {} => {}'.format(remote_dir_item, local_dir_item))
                        sftp.get(remote_dir_item, local_dir_item, preserve_mtime=True)
                        self.unprocessed_files_list.append(remote_dir_item)
                    files_synced += 1
            else:
                files_synced += self._sync_r(sftp, remote_dir_item, local_dir_item)
        return files_synced

    def _should_sync_file(self, sftp, remote_file_path, local_file_path):
        """
        If the remote_file should be synced - if it was not downloaded or it is out of sync with the remote version.
        :param sftp:                Connection to the sftp server.
        :param remote_file_path:    Remote file path.
        :param local_file_path:     Local file path.
        """
        if not os.path.exists(local_file_path):
            return True
        else:
            remote_attr = sftp.lstat(remote_file_path)
            local_stat = os.stat(local_file_path)
            return remote_attr.st_size != local_stat.st_size or remote_attr.st_mtime != local_stat.st_mtime

    @staticmethod
    def put_files(config, file_path):
        """Upload local files to SFTP server"""

        success = {}
        dest_path = ''
        local_file_name = ''
        try:
            local_file_name = file_path.replace("/", "|")
            dir_path = os.path.dirname(file_path)
            file_name = os.path.basename(file_path)

            dest_path = os.path.join(config['dest'], dir_path)
            local_file = open(local_file_name, "rb")

            sftp = FTPUtility.get_sftp_connection(config['host_name'], config['username'],
                                                config['password'])

            SFTPProcessor._mkdir_p(sftp, dest_path)

            sftp.put(local_file, preserve_mtime=True)

            sftp.close()
            local_file.close()
            success['status'] = Common.process_status_2.value
            success['msg'] = "Success"
            success['source_path'] = file_path
            success['dest_path'] = dest_path
        except Exception as ex:
            success['status'] = Common.process_status_1.value
            success['msg'] = str(ex)
            success['source_path'] = file_path
            success['dest_path'] = dest_path
        finally:
            FTPUtility.delete_file(local_file_name)
        return success

    @staticmethod
    def _mkdir_p(sftp, remote_directory):
        """Change to this directory, recursively making new folders if needed.
        Returns True if any folders were created."""
        if remote_directory == '/':
            # absolute path so change directory to root
            sftp.chdir('/')
            return
        if remote_directory == '':
            # top-level relative directory must exist
            return
        try:
            sftp.chdir(remote_directory)  # sub-directory exists
        except IOError:
            dirname, basename = os.path.split(remote_directory.rstrip('/'))
            SFTPProcessor._mkdir_p(sftp, dirname)  # make parent directories
            sftp.mkdir(basename)  # sub-directory missing, so created it
            sftp.chdir(basename)
            return True









"""Local file transfer"""

import logging.config
import os
import socket
import sys
import threading
import time

import boto3
from boto3.s3.transfer import S3Transfer
from botocore import exceptions

from datatransfer import constants
from datatransfer.util import Utility


class LocalDataTransfer:
    """Local file transfer"""

    def __init__(self):
        self.logger = logging.getLogger(
            constants.Constant.logger_module_name.value)

    @staticmethod
    def __get_files__(path):
        files = []
        if os.path.isdir(path):
            for entry in os.scandir(path):
                if entry.is_file():
                    files.append(entry.path)
        else:
            files.append(path)
        return files

    def upload(self, path):
        try:
            user_name = constants.SavedObj.config[constants.Config.client_credential_user.value]
            password = constants.SavedObj.config[constants.Config.client_credential_password.value]
            s3_conf = Utility.__authenticate_request__(user_name, password)
            access_key = s3_conf.get('access_key')
            secret_key = s3_conf.get('secret_key')
            bucket_name = s3_conf.get('bucket')

            transfer = S3Transfer(boto3.client('s3', aws_access_key_id=access_key,
                                               aws_secret_access_key=secret_key))
            files = self.__get_files__(path)
            for file in files:
                self.logger.info("start processing for %s", file)
                # ignore "." files
                if self.__ignore_file(file):
                    self.logger.info("ignoring %s", file)
                    continue
                # check file processing status
                if self.__is_file_processed__(file):
                    self.logger.info(
                        "skipping %s cause it has been processed already", file)
                    continue

                client_id = "2"
                date_format = s3_conf.get('namespace')[2:]
                # compress file
                compress_file = Utility.__compress_file__(file, "/tmp", constants.SavedObj.config[constants.Config.compression_type.value])
                # check configure limit
                if self.__is_s3_size_limit_exceeded__(compress_file, client_id,
                                                      access_key, secret_key,
                                                      bucket_name):
                    continue
                dir_name = os.path.basename(os.path.dirname(file))
                file_name = os.path.basename(compress_file)
                hostname_unixtime = socket.gethostname() + "_" + str(
                    int(time.time()))
                file_name = hostname_unixtime + "_" + file_name
                dest_path = client_id + "/" + dir_name + "/" + date_format + "/" + file_name

                # upload file
                transfer.upload_file(compress_file, bucket_name, dest_path,
                                     callback=ProgressPercentage(
                                         compress_file))
                # save file status
                self.__save_file_status__(file)
                self.logger.info("%s has been uploaded to S3", compress_file)

                # clean files
                if constants.SavedObj.config.get(
                        constants.Config.delete_file.value):
                    os.remove(file)
                    self.logger.info("delete file %s", file)
                    os.remove(compress_file)
                    self.logger.info("delete compressed file %s", compress_file)

        except FileNotFoundError as file_nf_err:
            self.logger.error(file_nf_err)
            sys.exit("File not found")
        except exceptions.ClientError as aws_err:
            self.logger.error(aws_err.response)
            sys.exit("AWS authentication exception")
        except Exception as ex:
            self.logger.error(ex)
            raise

    def __is_file_processed__(self, path):
        is_processed = False
        try:
            file = open(constants.SavedObj.config[constants.Config.file_process_log.value], 'r')
            lines = list(file)
            is_processed = lines.__contains__(path + "\n")
            file.close()
        except FileNotFoundError:
            self.logger.info("initialize process log")
            f = open(constants.SavedObj.config[constants.Config.file_process_log.value], "w+")
            f.close()

        return is_processed

    @staticmethod
    def __ignore_file(path):
        is_processed = False
        file_name = os.path.basename(path)
        if file_name.startswith("."):
            is_processed = True
        return is_processed


    def __is_s3_size_limit_exceeded__(self, path, client_id, access_key,
                                      secret_key, bucket_name):
        current_s3_size = self._s3_folder_size(client_id, access_key,
                                               secret_key, bucket_name)
        input_size = self.__input_path_size(path)
        if constants.Constant.default_s3_limit_mb.value < current_s3_size + input_size:
            self.logger.error(
                "Exceeding size limit. Max limit is %sMB. "
                "Current input size %sMB and used space is %sMB",
                constants.Constant.default_s3_limit_mb.value, input_size,
                current_s3_size)
            return True
        else:
            return False

    @staticmethod
    def _s3_folder_size(client_id, access_key, secret_key, bucket_name):
        session = boto3.Session(aws_access_key_id=access_key,
                                aws_secret_access_key=secret_key)
        s3 = session.resource('s3')
        total_size = 0
        for obj in s3.Bucket(bucket_name).objects.filter(Prefix=client_id):
            total_size += obj.size
        return int(total_size / 1024 / 1024)

    @staticmethod
    def __input_path_size(path):
        total_input_size = 0
        if os.path.isdir(path):
            total_input_size = sum(
                entry.stat().st_size for entry in os.scandir(path))
        else:
            total_input_size = os.stat(path).st_size
        return int(total_input_size / 1024 / 1024)

    def __save_file_status__(self, path):
        file = open(
            constants.SavedObj.config[constants.Config.file_process_log.value],
            'a+')
        file.write(path + "\n")
        self.logger.info("write the file processing status %s", path)
        file.close()


class ProgressPercentage:
    """File upload progress bar"""
    logger = logging.getLogger(constants.Constant.logger_module_name.value)

    def __init__(self, filename):
        self._filename = filename
        self._size = float(os.path.getsize(filename))
        self._seen_so_far = 0
        self._lock = threading.Lock()

    def __call__(self, bytes_amount):
        # To simplify we'll assume this is hooked up
        # to a single filename.
        with self._lock:
            self._seen_so_far += bytes_amount
            percentage = (self._seen_so_far / self._size) * 100
            self.logger.info(
                "%s  %s / %s  (%.2f%%)" % (
                    self._filename, self._seen_so_far, self._size,
                    percentage))



import gzip
import os
import shutil
from datatransfer.rest_client import RestClient
from datatransfer import constants


class Utility:


    @staticmethod
    def __is_file_compressed(filename):
        is_file_compressed = False
        if filename.endswith(".gz") or filename.endswith(".gzip") or filename.endswith(".tar") \
                or filename.endswith(".zip") or filename.endswith("bz2")\
                or filename.endswith("xz") or filename.endswith("rar"):
            is_file_compressed = True
        return is_file_compressed

    @staticmethod
    def __compress_file__(path,compressed_path_dir, compress_type):

        if Utility.__is_file_compressed(path):
            return path
        if compress_type == "gz":
            compressed_path = os.path.join(compressed_path_dir,
                                           os.path.basename(path) + ".gz")
            with open(path, 'rb') as f_in:
                with gzip.open(compressed_path, 'wb',
                               compresslevel=6) as f_out:
                    shutil.copyfileobj(f_in, f_out)
            return compressed_path
        else:
            return path

    @staticmethod
    def __authenticate_request__(user_name, password):

        token_res = RestClient.get(
            url=constants.Constant.score_token_url.value,
            auth={'user': user_name,
                  'password': password}).json()
        if token_res.get('status') != 'success':
            raise Exception("Unauthorized Request: %s" % str(token_res.get('detail')))
        token = token_res.get('token')
        s3_conf = RestClient.post(
            url=constants.Constant.score_s3_config_url.value,
            headers={'Authorization': 'Token ' + token},
            data={"token": token}).json()
        if token_res.get('status') != 'success':
            raise Exception("Unauthorized Request, Connect to admin")
        return s3_conf.get('data')



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='scorefastlib',
    version='0.1.3',
    license='',
    author='scorefast',
    author_email='ravisharma@scoredata.com',
    description='scorefast customer tool',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ScoreDataCorporation/scorefast-lib",
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
    packages=find_packages(),
)



# -*- coding: utf-8 -*-
import enum


class Constant(enum.Enum):
    """ constant """
    root_url = "https://console.scoredata.com"
    dataset_url = "{}/api/imported_data/".format(root_url)
    dataset_download_url = "{}/api/dataset/download/".format(root_url)
    model_download_url = "{}/api/model/download".format(root_url)
    model_url = "{}/api/model/".format(root_url)
    model_import_url = "{}/api/model/import/".format(root_url)
    user_summary_url = "{}/api/user/summary/".format(root_url)
    dataset_parse_url = "{}/api/dataset/parse/".format(root_url)
    dataset_import_url = "{}/api/imported_data/".format(root_url)
    project_url = "{}/api/project/".format(root_url)
    file_url = "{}/api/file/".format(root_url)
    
    s3_bucket = "scoredata-us-west1-data"
    s3_environment = "QA"
